function [ points ] = Picture_points( UAV_position, UAV_attitude, use_gim, gim_angle)
% Function that estimates the 4 corner points of a picture taken by the UAV
% at the given position and attitude. In this function, the cameras feild
% of view is 35x27 degrees

% coordinates in NED frame
x = 0;
y = 0;
z = - UAV_position(3);

UAV_NED = [x,y,z]';
C_cb = [0,0,(1/2)*z]';
c = abs(C_cb(3));

W = 35/2;
H = 27/2;
tW = tand(W);
tH = tand(H);

FS_cb = [c*tW, c*tH, c]';
FP_cb = [c*tW, -c*tH, c]';
RS_cb = [-c*tW, c*tH, c]';
RP_cb = [-c*tW, -c*tH, c]';

M = zeros(3,3);
if use_gim == 0
    M = NED2BODY(UAV_attitude(1),UAV_attitude(2),UAV_attitude(3));
else
    M = ROT_WITH_GIM(UAV_attitude(3),gim_angle);
end
d = 0.2;

r_co_cam_body = [0,0,d]';
r_co_cam_ned = M*r_co_cam_body;
rc = UAV_NED + r_co_cam_ned;

C_proj = M * C_cb + rc;
FS_proj = M * FS_cb + rc;
FP_proj = M * FP_cb + rc;
RS_proj = M * RS_cb + rc;
RP_proj = M * RP_cb + rc;

deltaC = C_proj-rc;
t = -rc(3)/deltaC(3);
cp = [rc(1)+deltaC(1)*t, rc(2)+deltaC(2)*t, rc(3)+deltaC(3)*t]';

deltaFS = FS_proj - rc;
t = -rc(3)/deltaFS(3);
fs = [rc(1)+deltaFS(1)*t,rc(2)+deltaFS(2)*t,rc(3)+deltaFS(3)*t]';

deltaFP = FP_proj - rc;
t = -rc(3)/deltaFP(3);
fp = [rc(1)+deltaFP(1)*t,rc(2)+deltaFP(2)*t,rc(3)+deltaFP(3)*t]';

deltaRS = RS_proj - rc;
t = -rc(3)/deltaRS(3);
rs = [rc(1)+deltaRS(1)*t,rc(2)+deltaRS(2)*t,rc(3)+deltaRS(3)*t]';

deltaRP = RP_proj - rc;
t = -rc(3)/deltaRP(3);
rp = [rc(1)+deltaRP(1)*t,rc(2)+deltaRP(2)*t,rc(3)+deltaRP(3)*t]';

% calculate the points GPS positions using the UAVs position as the
% reference point

r_fs = NED2GPS(fs,UAV_position);
r_fp = NED2GPS(fp,UAV_position);
r_rs = NED2GPS(rs,UAV_position);
r_rp = NED2GPS(rp,UAV_position);

points = [r_fs';
    r_fp';
    r_rs';
    r_rp'];
end

