/*
 * Below is the copyright agreement for IMCJava.
 * 
 * Copyright (c) 2010-2013, Laboratório de Sistemas e Tecnologia Subaquática
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     - Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     - Neither the names of IMC, LSTS, IMCJava nor the names of its 
 *       contributors may be used to endorse or promote products derived from 
 *       this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL LABORATORIO DE SISTEMAS E TECNOLOGIA SUBAQUATICA
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 *                                                                             $:
 */

// Source generated by IMCJava from IMC version 5.3.1
package pt.lsts.imc;

/**
 *  IMC Message ArduPilot State (1511)<br/>
 *  State of the ardupilot, including armed state, mode and other optional information<br/>
 */

public class ArduPilotState extends IMCMessage {

	public static final int ID_STATIC = 1511;

	public enum TYPE {
		GET(0),
		SET(1),
		SET_SUCCESS(2),
		GET_REPLY(3),
		STATE(4);

		protected long value;

		public long value() {
			return value;
		}

		TYPE(long value) {
			this.value = value;
		}
	}

	public ArduPilotState() {
		super(ID_STATIC);
	}

	public ArduPilotState(IMCDefinition defs) {
		super(defs, ID_STATIC);
	}

	public static ArduPilotState create(Object... values) {
		ArduPilotState m = new ArduPilotState();
		for (int i = 0; i < values.length-1; i+= 2)
			m.setValue(values[i].toString(), values[i+1]);
		return m;
	}

	public static ArduPilotState clone(IMCMessage msg) throws Exception {

		ArduPilotState m = new ArduPilotState();
		if (msg == null)
			return m;
		if(msg.definitions != m.definitions){
			msg = msg.cloneMessage();
			IMCUtil.updateMessage(msg, m.definitions);
		}
		else if (msg.getMgid()!=m.getMgid())
			throw new Exception("Argument "+msg.getAbbrev()+" is incompatible with message "+m.getAbbrev());

		m.getHeader().values.putAll(msg.getHeader().values);
		m.values.putAll(msg.values);
		return m;
	}

	public ArduPilotState(TYPE type, String mode, short armed, String other) {
		super(ID_STATIC);
		setType(type);
		if (mode != null)
			setMode(mode);
		setArmed(armed);
		if (other != null)
			setOther(other);
	}

	/**
	 *  Indicates if the message is a request or a reply to a<br/>
	 *  previous request. The <code>op*, *request_id* and *plan_id</code> fields<br/>
	 *  of a request will be echoed in one or more responses to that<br/>
	 *  request.<br/>
	 *  @return Type (enumerated) - uint8_t
	 */
	public TYPE getType() {
		try {
			TYPE o = TYPE.valueOf(getMessageType().getFieldPossibleValues("type").get(getLong("type")));
			return o;
		}
		catch (Exception e) {
			return null;
		}
	}

	/**
	 *  @return Mode - plaintext
	 */
	public String getMode() {
		return getString("mode");
	}

	/**
	 *  @return Armed - uint8_t
	 */
	public short getArmed() {
		return (short) getInteger("armed");
	}

	/**
	 *  @return other (tuplelist) - plaintext
	 */
	public java.util.LinkedHashMap<String, String> getOther() {
		return getTupleList("other");
	}

	/**
	 *  @param type Type (enumerated)
	 */
	public void setType(TYPE type) {
		values.put("type", type.value());
	}

	/**
	 *  @param type Type (as a String)
	 */
	public void setType(String type) {
		setValue("type", type);
	}

	/**
	 *  @param type Type (integer value)
	 */
	public void setType(short type) {
		setValue("type", type);
	}

	/**
	 *  @param mode Mode
	 */
	public void setMode(String mode) {
		values.put("mode", mode);
	}

	/**
	 *  @param armed Armed
	 */
	public void setArmed(short armed) {
		values.put("armed", armed);
	}

	/**
	 *  @param other other (tuplelist)
	 */
	public void setOther(java.util.LinkedHashMap<String, ?> other) {
		String val = encodeTupleList(other);
		values.put("other", val);
	}

	public void setOther(String other) {
		values.put("other", other);
	}

}
