/*
 * Below is the copyright agreement for IMCJava.
 * 
 * Copyright (c) 2010-2013, Laboratório de Sistemas e Tecnologia Subaquática
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     - Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     - Neither the names of IMC, LSTS, IMCJava nor the names of its 
 *       contributors may be used to endorse or promote products derived from 
 *       this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL LABORATORIO DE SISTEMAS E TECNOLOGIA SUBAQUATICA
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 *                                                                             $:
 */

// Source generated by IMCJava from IMC version 5.3.1
package pt.lsts.imc;

/**
 *  IMC Message Formation Control Parameters (820)<br/>
 *  Formation controller paramenters, as: trajectory gains, control boundary layer thickness, and formation shape gains.<br/>
 */

public class FormCtrlParam extends IMCMessage {

	public static final int ID_STATIC = 820;

	public enum ACTION {
		REQ(0),
		SET(1),
		REP(2);

		protected long value;

		public long value() {
			return value;
		}

		ACTION(long value) {
			this.value = value;
		}
	}

	public FormCtrlParam() {
		super(ID_STATIC);
	}

	public FormCtrlParam(IMCDefinition defs) {
		super(defs, ID_STATIC);
	}

	public static FormCtrlParam create(Object... values) {
		FormCtrlParam m = new FormCtrlParam();
		for (int i = 0; i < values.length-1; i+= 2)
			m.setValue(values[i].toString(), values[i+1]);
		return m;
	}

	public static FormCtrlParam clone(IMCMessage msg) throws Exception {

		FormCtrlParam m = new FormCtrlParam();
		if (msg == null)
			return m;
		if(msg.definitions != m.definitions){
			msg = msg.cloneMessage();
			IMCUtil.updateMessage(msg, m.definitions);
		}
		else if (msg.getMgid()!=m.getMgid())
			throw new Exception("Argument "+msg.getAbbrev()+" is incompatible with message "+m.getAbbrev());

		m.getHeader().values.putAll(msg.getHeader().values);
		m.values.putAll(msg.values);
		return m;
	}

	public FormCtrlParam(ACTION Action, long LonGain, long LatGain, long BondThick, long LeadGain, long DeconflGain) {
		super(ID_STATIC);
		setAction(Action);
		setLonGain(LonGain);
		setLatGain(LatGain);
		setBondThick(BondThick);
		setLeadGain(LeadGain);
		setDeconflGain(DeconflGain);
	}

	/**
	 *  Action on the vehicle formation control parameters.<br/>
	 *  @return Action (enumerated) - uint8_t
	 */
	public ACTION getAction() {
		try {
			ACTION o = ACTION.valueOf(getMessageType().getFieldPossibleValues("Action").get(getLong("Action")));
			return o;
		}
		catch (Exception e) {
			return null;
		}
	}

	/**
	 *  @return Longitudinal Gain - uint32_t
	 */
	public long getLonGain() {
		return getLong("LonGain");
	}

	/**
	 *  @return Lateral Gain - uint32_t
	 */
	public long getLatGain() {
		return getLong("LatGain");
	}

	/**
	 *  @return Boundary Layer Thickness - uint32_t
	 */
	public long getBondThick() {
		return getLong("BondThick");
	}

	/**
	 *  @return Leader Gain - uint32_t
	 */
	public long getLeadGain() {
		return getLong("LeadGain");
	}

	/**
	 *  @return Deconfliction Gain - uint32_t
	 */
	public long getDeconflGain() {
		return getLong("DeconflGain");
	}

	/**
	 *  @param Action Action (enumerated)
	 */
	public void setAction(ACTION Action) {
		values.put("Action", Action.value());
	}

	/**
	 *  @param Action Action (as a String)
	 */
	public void setAction(String Action) {
		setValue("Action", Action);
	}

	/**
	 *  @param Action Action (integer value)
	 */
	public void setAction(short Action) {
		setValue("Action", Action);
	}

	/**
	 *  @param LonGain Longitudinal Gain
	 */
	public void setLonGain(long LonGain) {
		values.put("LonGain", LonGain);
	}

	/**
	 *  @param LatGain Lateral Gain
	 */
	public void setLatGain(long LatGain) {
		values.put("LatGain", LatGain);
	}

	/**
	 *  @param BondThick Boundary Layer Thickness
	 */
	public void setBondThick(long BondThick) {
		values.put("BondThick", BondThick);
	}

	/**
	 *  @param LeadGain Leader Gain
	 */
	public void setLeadGain(long LeadGain) {
		values.put("LeadGain", LeadGain);
	}

	/**
	 *  @param DeconflGain Deconfliction Gain
	 */
	public void setDeconflGain(long DeconflGain) {
		values.put("DeconflGain", DeconflGain);
	}

}
