/*
 * Below is the copyright agreement for IMCJava.
 * 
 * Copyright (c) 2010-2013, Laboratório de Sistemas e Tecnologia Subaquática
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     - Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     - Neither the names of IMC, LSTS, IMCJava nor the names of its 
 *       contributors may be used to endorse or promote products derived from 
 *       this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL LABORATORIO DE SISTEMAS E TECNOLOGIA SUBAQUATICA
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 *                                                                             $:
 */

// Source generated by IMCJava from IMC version 5.3.1
package pt.lsts.imc;

/**
 *  IMC Message LBL Beacon Configuration (202)<br/>
 *  Position and configuration of an LBL transponder (beacon).<br/>
 */

public class LblBeacon extends IMCMessage {

	public static final int ID_STATIC = 202;

	public LblBeacon() {
		super(ID_STATIC);
	}

	public LblBeacon(IMCDefinition defs) {
		super(defs, ID_STATIC);
	}

	public static LblBeacon create(Object... values) {
		LblBeacon m = new LblBeacon();
		for (int i = 0; i < values.length-1; i+= 2)
			m.setValue(values[i].toString(), values[i+1]);
		return m;
	}

	public static LblBeacon clone(IMCMessage msg) throws Exception {

		LblBeacon m = new LblBeacon();
		if (msg == null)
			return m;
		if(msg.definitions != m.definitions){
			msg = msg.cloneMessage();
			IMCUtil.updateMessage(msg, m.definitions);
		}
		else if (msg.getMgid()!=m.getMgid())
			throw new Exception("Argument "+msg.getAbbrev()+" is incompatible with message "+m.getAbbrev());

		m.getHeader().values.putAll(msg.getHeader().values);
		m.values.putAll(msg.values);
		return m;
	}

	public LblBeacon(String beacon, double lat, double lon, float depth, short query_channel, short reply_channel, short transponder_delay) {
		super(ID_STATIC);
		if (beacon != null)
			setBeacon(beacon);
		setLat(lat);
		setLon(lon);
		setDepth(depth);
		setQueryChannel(query_channel);
		setReplyChannel(reply_channel);
		setTransponderDelay(transponder_delay);
	}

	/**
	 *  @return Beacon Name - plaintext
	 */
	public String getBeacon() {
		return getString("beacon");
	}

	/**
	 *  @return Latitude WGS-84 (rad) - fp64_t
	 */
	public double getLat() {
		return getDouble("lat");
	}

	/**
	 *  @return Longitude WGS-84 (rad) - fp64_t
	 */
	public double getLon() {
		return getDouble("lon");
	}

	/**
	 *  @return Depth (m) - fp32_t
	 */
	public double getDepth() {
		return getDouble("depth");
	}

	/**
	 *  @return Interrogation channel - uint8_t
	 */
	public short getQueryChannel() {
		return (short) getInteger("query_channel");
	}

	/**
	 *  @return Reply channel - uint8_t
	 */
	public short getReplyChannel() {
		return (short) getInteger("reply_channel");
	}

	/**
	 *  @return Transponder delay (ms) - uint8_t
	 */
	public short getTransponderDelay() {
		return (short) getInteger("transponder_delay");
	}

	/**
	 *  @param beacon Beacon Name
	 */
	public void setBeacon(String beacon) {
		values.put("beacon", beacon);
	}

	/**
	 *  @param lat Latitude WGS-84 (rad)
	 */
	public void setLat(double lat) {
		values.put("lat", lat);
	}

	/**
	 *  @param lon Longitude WGS-84 (rad)
	 */
	public void setLon(double lon) {
		values.put("lon", lon);
	}

	/**
	 *  @param depth Depth (m)
	 */
	public void setDepth(double depth) {
		values.put("depth", depth);
	}

	/**
	 *  @param query_channel Interrogation channel
	 */
	public void setQueryChannel(short query_channel) {
		values.put("query_channel", query_channel);
	}

	/**
	 *  @param reply_channel Reply channel
	 */
	public void setReplyChannel(short reply_channel) {
		values.put("reply_channel", reply_channel);
	}

	/**
	 *  @param transponder_delay Transponder delay (ms)
	 */
	public void setTransponderDelay(short transponder_delay) {
		values.put("transponder_delay", transponder_delay);
	}

}
