/*
 * Below is the copyright agreement for IMCJava.
 * 
 * Copyright (c) 2010-2013, Laboratório de Sistemas e Tecnologia Subaquática
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     - Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     - Neither the names of IMC, LSTS, IMCJava nor the names of its 
 *       contributors may be used to endorse or promote products derived from 
 *       this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL LABORATORIO DE SISTEMAS E TECNOLOGIA SUBAQUATICA
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 * $Id::                                                                       $:
 */
package pt.lsts.imc.types;

import java.util.LinkedHashMap;

import pt.lsts.imc.Heartbeat;
import pt.lsts.imc.IMCDefinition;
import pt.lsts.imc.IMCMessage;

public class ImcTranslation {

    protected static IMessageAdapter[] adapters = new IMessageAdapter[] {
        new LblConfigAdapter(),
        new PlanSpecificationAdapter(),
        new EstimatedStateAdapter(),
        new SonarDataAdapter(),
        new SonarConfigAdapter(),
        new EulerAnglesAdapter() 
     };
    
    protected static LinkedHashMap<String, IMessageAdapter> translators = new LinkedHashMap<String, IMessageAdapter>();
    
    static {
        for (IMessageAdapter adapter : adapters) {
            for (String msg : adapter.getCompatibleMessages())
                translators.put(msg, adapter);
        }
    }
    
    public static IMCMessage translate(IMCMessage msg, IMCDefinition destination) {
        if (msg.getMessageType().getImcVersion().equals(destination.getVersion()))
            return msg;
        
        if (translators.containsKey(msg.getAbbrev())) {
            IMessageAdapter translator = translators.get(msg.getAbbrev()); 
            translator.setData(msg);
            msg = translator.getData(destination);            
        }
        
        return msg;
    }
    
    public static void main(String[] args) {
        Heartbeat hb = new Heartbeat();
        ImcTranslation.translate(hb, IMCDefinition.getImc4Instance());
    }
}
