/*
 * Copyright (c) 2004-2014 Universidade do Porto - Faculdade de Engenharia
 * Laboratório de Sistemas e Tecnologia Subaquática (LSTS)
 * All rights reserved.
 * Rua Dr. Roberto Frias s/n, sala I203, 4200-465 Porto, Portugal
 *
 * This file is part of Neptus, Command and Control Framework.
 *
 * Commercial Licence Usage
 * Licencees holding valid commercial Neptus licences may use this file
 * in accordance with the commercial licence agreement provided with the
 * Software or, alternatively, in accordance with the terms contained in a
 * written agreement between you and Universidade do Porto. For licensing
 * terms, conditions, and further information contact lsts@fe.up.pt.
 *
 * European Union Public Licence - EUPL v.1.1 Usage
 * Alternatively, this file may be used under the terms of the EUPL,
 * Version 1.1 only (the "Licence"), appearing in the file LICENSE.md
 * included in the packaging of this file. You may not use this work
 * except in compliance with the Licence. Unless required by applicable
 * law or agreed to in writing, software distributed under the Licence is
 * distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF
 * ANY KIND, either express or implied. See the Licence for the specific
 * language governing permissions and limitations at
 * https://www.lsts.pt/neptus/licence.
 *
 * For more information please see <http://lsts.fe.up.pt/neptus>.
 *
 * Author: havard
 * Mar 5, 2014
 */
package info.havard.PictureOutlineOnMap;

import pt.lsts.neptus.types.coord.LocationType;

/**
 * @author havard
 * A static function that creates a LocationType object when given a point in distance N,E from another LocationType object
 * based on http://www.mathworks.se/help/aeroblks/flatearthtolla.html
 */
public class MakeLocation {

    public static LocationType ConvertFromNED(LocationType ref, double North, double East)
    {
        double flattening = 1/298.257223563;
        double R_eq = 6378137.0;
        
        double sinlat = Math.sin(ref.getLatitudeRads());
        
        
        double R_N = R_eq / Math.sqrt(1-(2*flattening-(flattening*flattening))*Math.pow(sinlat,2));
        double R_M = R_N*((1-(2*flattening-(flattening*flattening)))/(1-(2*flattening-(flattening*flattening))*Math.pow(sinlat,2)));
        
        double dlat = Math.atan(1/R_M)*North;
        double dlong = Math.atan(1/(R_N * Math.cos(ref.getLatitudeRads())))*East;
        
        double lat = ref.getLatitudeRads() + dlat;
        double lon = ref.getLongitudeRads() + dlong;
        

        LocationType C =  new LocationType(Math.toDegrees(lat),Math.toDegrees(lon));
        
        return C;
    }
public static void main(String[] args) {
    
    // Test of the MakeLocation-function
    // Output OK
    
    //LocationType L = new LocationType(41.2,-8.7);
    //LocationType L2 = MakeLocation.ConvertFromNED(L, 10, 10);
    
    //System.out.println("New point: " + L2.getLatitudeDegs() + "  " +  L2.getLongitudeDegs());
}
}
