/*
 * Copyright (c) 2004-2014 Universidade do Porto - Faculdade de Engenharia
 * Laboratório de Sistemas e Tecnologia Subaquática (LSTS)
 * All rights reserved.
 * Rua Dr. Roberto Frias s/n, sala I203, 4200-465 Porto, Portugal
 *
 * This file is part of Neptus, Command and Control Framework.
 *
 * Commercial Licence Usage
 * Licencees holding valid commercial Neptus licences may use this file
 * in accordance with the commercial licence agreement provided with the
 * Software or, alternatively, in accordance with the terms contained in a
 * written agreement between you and Universidade do Porto. For licensing
 * terms, conditions, and further information contact lsts@fe.up.pt.
 *
 * European Union Public Licence - EUPL v.1.1 Usage
 * Alternatively, this file may be used under the terms of the EUPL,
 * Version 1.1 only (the "Licence"), appearing in the file LICENSE.md
 * included in the packaging of this file. You may not use this work
 * except in compliance with the Licence. Unless required by applicable
 * law or agreed to in writing, software distributed under the Licence is
 * distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF
 * ANY KIND, either express or implied. See the Licence for the specific
 * language governing permissions and limitations at
 * https://www.lsts.pt/neptus/licence.
 *
 * For more information please see <http://lsts.fe.up.pt/neptus>.
 *
 * Author: havard
 * Feb 25, 2014
 */
package info.havard.PictureOutlineOnMap;


import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;

import pt.lsts.imc.IMCMessage;
import pt.lsts.neptus.comm.manager.imc.ImcId16;
import pt.lsts.neptus.console.ConsoleLayout;
import pt.lsts.neptus.i18n.I18n;
import pt.lsts.neptus.plugins.MultiSystemIMCMessageListener;
import pt.lsts.neptus.plugins.PluginDescription;
import pt.lsts.neptus.plugins.SimpleRendererInteraction;
import pt.lsts.neptus.plugins.PluginDescription.CATEGORY;
import pt.lsts.neptus.renderer2d.StateRenderer2D;
import pt.lsts.neptus.types.coord.LocationType;
import pt.lsts.neptus.util.ImageUtils;
import pt.lsts.neptus.util.MathMiscUtils;


/**
 * @author havard
 *
 */
@PluginDescription(author="havard", category= CATEGORY.UNSORTED, name="Picture Outline on Map")
public class MarkPicture extends SimpleRendererInteraction implements ComponentListener{

    /**
     * 
     */
    private static final long serialVersionUID = -9086325257916748858L;
    private List<PictureCorners> points = new ArrayList<PictureCorners>();
    private List<PictureCorners> all_points = new ArrayList<PictureCorners>();
    private double lat = 0;
    private double lon = 0;
    private double roll = 0;
    private double alt = 1;
    private double pitch = 0;
    private double yaw = 0;
    private double x = 0;
    private double y = 0;
    private double z = 0;
    private Object synchObject = new Object();
    private Point2D fs;
    private Point2D fp;
    private Point2D rs;
    private Point2D rp;
    private PictureCorners p;
    private Iterator<PictureCorners> iter;
    public JPopupMenu popup;
    private Color col = Color.ORANGE;

    private PictureCorners pc;
    
    private boolean CameraGimbal = true;
    // true - without gimbal
    // false - without gimbal
    private double gimbalAngle = 45;


    //listener object which allows the panel to tap into the various IMC messages
    private MultiSystemIMCMessageListener listener = new MultiSystemIMCMessageListener(this.getClass().getSimpleName()
            + " [" + Integer.toHexString(hashCode()) + "]") {

        @Override
        public void messageArrived(ImcId16 id, IMCMessage msg) {
            
            
            //Check if the message is coming from a UAV. Only if it is, do something
            //if (ImcSystemsHolder.lookupSystem(id).getTypeVehicle().name().equalsIgnoreCase("UAV")){
            
            try
            {

                lat = Math.toDegrees(msg.getDouble("lat"));
                lon = Math.toDegrees(msg.getDouble("lon"));
                alt = msg.getDouble("alt");
             
                
                roll = Math.toDegrees(msg.getDouble("phi"));
                pitch = Math.toDegrees(msg.getDouble("theta"));
                yaw = Math.toDegrees(msg.getDouble("psi"));
                
                x = msg.getDouble("x");
                y = msg.getDouble("y");
                z = msg.getDouble("z");
                //alt = alt-z;  ??
                
                LocationType ref = new LocationType(lat,lon);
                LocationType loc1 = MakeLocation.ConvertFromNED(ref, x, y);

                double[] p = {loc1.getLatitudeDegs(),loc1.getLongitudeDegs(),alt};
                double[] a = {roll,pitch,yaw};
                Matrix UAV_pos = Matrix.columnVector(p);
                Matrix UAV_att = Matrix.columnVector(a);
            
                pc = new PictureCorners(UAV_pos,UAV_att,35,27,CameraGimbal,gimbalAngle);
                System.out.println(pc.getFrontStarboard().getLatitudeDegs() + " " + pc.getFrontStarboard().getLongitudeDegs());
                synchronized(synchObject)
                {
                    points.add(pc);
                }
                repaint();
            }
            catch (Exception e)
            {
                // empty by design
                // if one does not work, skip.
            }                  
        }
    };
    
    
    /**
     * @param console
     */
    public MarkPicture(ConsoleLayout console) {
        super(console);
        
    }


    /* (non-Javadoc)
     * @see pt.lsts.neptus.renderer2d.StateRendererInteraction#isExclusive()
     */
    @Override
    public boolean isExclusive() {
        // TODO Auto-generated method stub
        return false;
    }

    /* (non-Javadoc)
     * @see pt.lsts.neptus.console.ConsolePanel#cleanSubPanel()
     */
    @Override
    public void cleanSubPanel() {
        // TODO Auto-generated method stub
        //System.out.println("This ends!");

    }

    /* (non-Javadoc)
     * @see pt.lsts.neptus.console.ConsolePanel#initSubPanel()
     */
    @Override
    public void initSubPanel() {
        // TODO Auto-generated method stub
        //System.out.println("This works!");   
        
    }

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
     */
    @Override
    public void componentHidden(ComponentEvent arg0) {
        // TODO Auto-generated method stub
        
    }

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
     */
    @Override
    public void componentMoved(ComponentEvent arg0) {
        // TODO Auto-generated method stub
        
    }

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
     */
    @Override
    public void componentResized(ComponentEvent arg0) {
        // TODO Auto-generated method stub
        
    }

    /* (non-Javadoc)
     * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
     */
    @Override
    public void componentShown(ComponentEvent arg0) {
        // TODO Auto-generated method stub
        
    }
    @Override
    public void paint(Graphics2D g, StateRenderer2D renderer)
    {

        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        synchronized(synchObject)
        {
            iter = points.iterator();
            while (iter.hasNext())
            {
                p = iter.next();
                try
                {
                    // convert the GPS_points to screen points
                    fs = renderer.getScreenPosition(p.getFrontStarboard());
                    fp = renderer.getScreenPosition(p.getFrontPort());
                    rs = renderer.getScreenPosition(p.getRearStarboard());
                    rp = renderer.getScreenPosition(p.getRearPort());
                    // create Shape to draw on map
                    Path2D path = new Path2D.Double();
                    path.moveTo(fs.getX(), fs.getY());
                    path.lineTo(fp.getX(), fp.getY());
                    path.lineTo(rp.getX(), rp.getY());
                    path.lineTo(rs.getX(), rs.getY());
                    path.closePath();
            
                    g.setColor(col);
                    g.draw(path);
                    g.fill(path);
                }
                catch (Exception e)
                {
                    // empty by design
                }
            }
        }

    }
    @Override
    public void mouseClicked(MouseEvent event, StateRenderer2D source) 
    {
        //final StateRenderer2D renderer = source;
        final Point2D mousePoint = event.getPoint();
        
        if (event.getButton() == MouseEvent.BUTTON2 ||event.getButton() == MouseEvent.BUTTON1 ) 
        {
            popup = new JPopupMenu();
            
            AbstractAction start_a = new AbstractAction(I18n.text("Start drawing")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {

                 // sets up the listener to listen to all vehicles
                    listener.setSystemToListen();
                    // Listen to estimated state message
                    String[] messages = {"EstimatedState"};
                    listener.setMessagesToListen(messages);
                    System.out.println("START");
                    
                    start_listener();
                    
                }
            };
            start_a.putValue(AbstractAction.SMALL_ICON, new ImageIcon(ImageUtils.getImage("images/menus/restart.png")));
            popup.add(start_a);
            
            
            AbstractAction del_draw = new AbstractAction(I18n.text("Permanently delete all drawings")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {

                    all_points.clear();
                    synchronized(synchObject)
                    {
                        points.clear();
                    }
                    
                }
            };
            del_draw.putValue(AbstractAction.SMALL_ICON, new ImageIcon(ImageUtils.getImage("images/menus/editdelete.png")));
            popup.add(del_draw);
            
            AbstractAction clear_draw = new AbstractAction(I18n.text("Clear drawings from map")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {
                    //CoordinateUtil.copyToClipboard(renderer.getRealWorldLocation(mousePoint));
                    all_points.addAll(points);
                    synchronized(synchObject)
                    {
                        points.clear();
                    }
                    
                }
            };
            clear_draw.putValue(AbstractAction.SMALL_ICON, new ImageIcon(ImageUtils.getImage("images/menus/edit.png")));
            popup.add(clear_draw);
            
            AbstractAction add_draw = new AbstractAction(I18n.text("Add all drawings to map")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {
                    all_points.addAll(points);
                    synchronized(synchObject)
                    {
                        points.clear();
                        points.addAll(all_points);
                    }
                    
                }
            };
            add_draw.putValue(AbstractAction.SMALL_ICON, new ImageIcon(ImageUtils.getImage("images/menus/editcopy.png")));
            popup.add(add_draw);
            
            AbstractAction change_col1 = new AbstractAction(I18n.text("Change color to red")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {
                    col = Color.RED;
                    
                }
            };
            change_col1.putValue(AbstractAction.SMALL_ICON, new ImageIcon(ImageUtils.getImage("images/menus/editcopy.png")));
            popup.add(change_col1);
            
            AbstractAction change_col2 = new AbstractAction(I18n.text("Change color to orange")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {
                    col = Color.ORANGE;
                    
                }
            };
            change_col2.putValue(AbstractAction.SMALL_ICON, new ImageIcon(ImageUtils.getImage("images/menus/editcopy.png")));
            popup.add(change_col2);
            
            AbstractAction change_gim1 = new AbstractAction(I18n.text("Switch to camera with gimbal")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {
                    CameraGimbal = true;
                }
            };
            popup.add(change_gim1);
            
            AbstractAction change_gim2 = new AbstractAction(I18n.text("Switch to camera without gimbal")) {
                private static final long serialVersionUID = 1L;

                @Override
                public void actionPerformed(ActionEvent e) {
                    CameraGimbal = false;
                }
            };
            popup.add(change_gim2);
                    
               
            
            popup.show(source, (int) mousePoint.getX(), (int) mousePoint.getY());

            
        }
    }


    /**
     * 
     */
    protected void start_listener() {
        this.addComponentListener(this);
        
    }
    protected void stop_listener() {
                listener.clean();
        
    }
    

}
