/*
 * Copyright (c) 2004-2014 Universidade do Porto - Faculdade de Engenharia
 * Laboratório de Sistemas e Tecnologia Subaquática (LSTS)
 * All rights reserved.
 * Rua Dr. Roberto Frias s/n, sala I203, 4200-465 Porto, Portugal
 *
 * This file is part of Neptus, Command and Control Framework.
 *
 * Commercial Licence Usage
 * Licencees holding valid commercial Neptus licences may use this file
 * in accordance with the commercial licence agreement provided with the
 * Software or, alternatively, in accordance with the terms contained in a
 * written agreement between you and Universidade do Porto. For licensing
 * terms, conditions, and further information contact lsts@fe.up.pt.
 *
 * European Union Public Licence - EUPL v.1.1 Usage
 * Alternatively, this file may be used under the terms of the EUPL,
 * Version 1.1 only (the "Licence"), appearing in the file LICENSE.md
 * included in the packaging of this file. You may not use this work
 * except in compliance with the Licence. Unless required by applicable
 * law or agreed to in writing, software distributed under the Licence is
 * distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF
 * ANY KIND, either express or implied. See the Licence for the specific
 * language governing permissions and limitations at
 * https://www.lsts.pt/neptus/licence.
 *
 * For more information please see <http://lsts.fe.up.pt/neptus>.
 *
 * Author: havard
 * Mar 5, 2014
 */
package info.havard.PictureOutlineOnMap;

import pt.lsts.neptus.types.coord.LocationType;

/**
 * @author havard
 *
 */
public class PictureCorners {
    private int picture_H;
    private int picture_W;
    private LocationType FS;
    private LocationType FP;
    private LocationType RS;
    private LocationType RP;
    
    public int getPicture_H() {
        return picture_H;
    }
    public void setPicture_H(int picture_H) {
        this.picture_H = picture_H;
    }
    public int getPicture_W() {
        return picture_W;
    }
    public void setPicture_W(int picture_W) {
        this.picture_W = picture_W;
    }
    public LocationType getFrontStarboard()
    {
        return FS;
    }
    public LocationType getFrontPort()
    {
        return FP;
    }
    public LocationType getRearStarboard()
    {
        return RS;
    }
    public LocationType getRearPort()
    {
        return RP;
    }
    public PictureCorners(Matrix UAV_position, Matrix UAV_attitude, int H, int W, boolean gimbal, double gim_angle) // two 3x1 matrices
    {
        this.setPicture_H(H);
        this.setPicture_W(W);
        
        double h = (double)this.getPicture_H()/2.0;
        double w = (double)this.getPicture_W()/2.0;
        
        double tH = Math.tan(Math.toRadians(h));
        double tW = Math.tan(Math.toRadians(w));
        
        // uav coordinates in the NED-frame
        double x = 0;
        double y = 0;
        double z = -UAV_position.getValue(3, 1);
        
        double[] ned = {x,y,z};
        Matrix UAV_NED = Matrix.columnVector(ned);
        
        
        // coordinates of the a frame halfway to the ground
        double[] c_cb = {0,0,(0.5)*z};
        Matrix C_CB = Matrix.columnVector(c_cb);
        double c = Math.abs(C_CB.getValue(3, 1));
        
        // point Front Starboard
        double[] fs = {c*tW,c*tH,c};
        Matrix FS_CB = Matrix.columnVector(fs);
        //FS_CB.show();
        
        // point Front Port
        double[] fp = {c*tW,-c*tH,c};
        Matrix FP_CB = Matrix.columnVector(fp);
        
        // point Rear Starboard
        double[] rs = {-c*tW,c*tH,c};
        Matrix RS_CB = Matrix.columnVector(rs);
        
        // point Rear Port
        double[] rp = {-c*tW,-c*tH,c};
        Matrix RP_CB = Matrix.columnVector(rp);
        Matrix T = null;
        if (gimbal)
        {
            T = RotMatrixWithGimbal.RotMatrix(UAV_attitude.getValue(3,1), gim_angle);
        }
        else
        {
            // Rotation from NED to BODY frame
            T = NED2BODY_RotationMatrix.NED2BODY(UAV_attitude.getValue(1,1), UAV_attitude.getValue(2,1), UAV_attitude.getValue(3,1));
            //T.show();
        }
        // compute the cameras position in NED frame
        double[] r = {0,0,0.20};    // distance from CO to the camera
        Matrix r_cam = Matrix.columnVector(r);
        Matrix r_cam_ned = T.times(r_cam);
        // Cameras position in NED frame
        Matrix rc = UAV_NED.plus(r_cam_ned);
        //rc.show();
        
        // The frame's corners in the ned frame, with adjustments for the UAVs attitude
        Matrix FS_proj = T.times(FS_CB).plus(rc);
        //FS_proj.show();
        Matrix FP_proj = T.times(FP_CB).plus(rc);
        Matrix RS_proj = T.times(RS_CB).plus(rc);
        Matrix RP_proj = T.times(RP_CB).plus(rc);
        
        // stigningstall til linjen mellom bakken og frame
        Matrix deltaFS = FS_proj.minus(rc);
        //deltaFS.show();
        double t_fs = -rc.getValue(3, 1) / deltaFS.getValue(3, 1);
        Matrix pointFS = rc.plus(deltaFS.timesConstant(t_fs));
        //pointFS.show();
                        
        Matrix deltaFP = FP_proj.minus(rc);
        double t_fp = -rc.getValue(3, 1) / deltaFP.getValue(3, 1);
        Matrix pointFP = rc.plus(deltaFP.timesConstant(t_fp));
        
        Matrix deltaRS = RS_proj.minus(rc);
        double t_rs = -rc.getValue(3, 1) / deltaRS.getValue(3, 1);
        Matrix pointRS = rc.plus(deltaRS.timesConstant(t_rs));
        
        Matrix deltaRP = RP_proj.minus(rc);
        double t_rp = -rc.getValue(3, 1) / deltaRP.getValue(3, 1);
        Matrix pointRP = rc.plus(deltaRP.timesConstant(t_rp));
        
        // Convert to GPS coordinates
        LocationType uav = new LocationType(UAV_position.getValue(1,1),UAV_position.getValue(2,1));
        
        this.FS = MakeLocation.ConvertFromNED(uav, pointFS.getValue(1,1), pointFS.getValue(2, 1));
        this.FP = MakeLocation.ConvertFromNED(uav, pointFP.getValue(1,1), pointFP.getValue(2, 1));
        this.RS = MakeLocation.ConvertFromNED(uav, pointRS.getValue(1,1), pointRS.getValue(2, 1));
        this.RP = MakeLocation.ConvertFromNED(uav, pointRP.getValue(1,1), pointRP.getValue(2, 1));

    }


}